@@ -2,9 +2,57 @@ use blsttc::{SecretKeySet, SecretKeyShare};
 use rand::{prelude::StdRng, Rng, SeedableRng};
 
 mod handover_net;
-use handover_net::{DummyProposal, Net};
+use handover_net::{Net, Packet};
 use sn_membership::{Ballot, Error, Handover, Result, SignedVote, Vote};
 
+#[test]
+fn test_handover_one_faulty_node_and_many_packet_drops() {
+    // make network of 5 elders with one segregated (his network is really bad)
+    let mut rng = StdRng::from_seed([0u8; 32]);
+    let mut net = Net::with_procs(3, 5, &mut rng);
+    let segregated_elder = net.procs.pop().unwrap();
+
+    // p0 is a bad node and the network is really bad for the segregated elder so he's not connected yet
+    // p0 makes 2 proposals:
+    // - 1 for the elders to see
+    // - 4 for the segregated elder
+    let p0 = net.procs[0].id();
+    let vote = net.procs[0].propose(1).unwrap();
+    net.broadcast(p0, vote);
+    net.drain_queued_packets().unwrap();
+    assert!(net.packets.is_empty());
+
+    // by the time everyone agreed on smth segregated_elder is back online and receives the bad vote
+    let bad_vote = net.procs[0]
+        .sign_vote(Vote {
+            gen: 0,
+            ballot: Ballot::Propose(4),
+            faults: Default::default(),
+        })
+        .unwrap();
+    net.enqueue_packets([Packet {
+        source: p0,
+        dest: segregated_elder.id(),
+        vote: bad_vote,
+    }]);
+    net.procs.push(segregated_elder);
+    net.drain_queued_packets().unwrap();
+
+    net.generate_msc("handover_one_faulty_node_and_many_packet_drops.msc")
+        .unwrap();
+
+    // since everyone agreed already they can't change their votes
+    // they have reached consensus
+    let first_voters_value = net.consensus_value(0);
+    for i in 0..5 {
+        let decision = net.consensus_value(i);
+        println!("[TEST] checking voter {i}'s consensus value: {decision:?}");
+        assert_eq!(decision, first_voters_value);
+    }
+
+    assert_eq!(first_voters_value, Some(1));
+}
+
 #[test]
 fn test_handover_reject_voter_changing_proposal_when_one_is_in_progress() -> Result<()> {
     let mut rng = StdRng::from_seed([0u8; 32]);
@@ -19,8 +67,8 @@ fn test_handover_reject_voter_changing_proposal_when_one_is_in_progress() -> Res
     proc.propose(111)?;
 
     assert!(matches!(
-        dbg!(proc.propose(222)),
-        Err(Error::ExistingVoteIncompatibleWithNewVote { .. })
+        proc.propose(222),
+        Err(Error::AttemptedFaultyProposal)
     ));
     Ok(())
 }
@@ -57,7 +105,7 @@ fn test_handover_handle_vote_rejects_packet_from_bad_gen() {
 
     // one elder votes with a different generation
     net.procs[1].gen = 401;
-    let vote = net.procs[1].propose(DummyProposal(rng.gen())).unwrap();
+    let vote = net.procs[1].propose(rng.gen()).unwrap();
 
     // make sure the other elder rejects that vote
     assert!(matches!(
@@ -84,7 +132,11 @@ fn test_handover_reject_votes_with_invalid_signatures() -> Result<()> {
     let voter = 1;
     let bytes = bincode::serialize(&(&ballot, &gen))?;
     let sig = rng.gen::<SecretKeyShare>().sign(&bytes);
-    let vote = Vote { gen, ballot };
+    let vote = Vote {
+        gen,
+        ballot,
+        faults: Default::default(),
+    };
     let resp = proc.handle_signed_vote(SignedVote { vote, voter, sig });
 
     assert!(resp.is_err());
@@ -97,7 +149,7 @@ fn test_handover_reject_votes_with_invalid_signatures() -> Result<()> {
 fn test_handover_split_vote() -> eyre::Result<()> {
     let mut rng = StdRng::from_seed([0u8; 32]);
     for nprocs in 1..7 {
-        println!("[TEST] testing with {} elders", nprocs);
+        println!("[TEST] testing with {nprocs} elders");
 
         // make network of nprocs elders
         let mut net = Net::with_procs((nprocs * 2 + 2) / 3, nprocs, &mut rng);
@@ -105,7 +157,7 @@ fn test_handover_split_vote() -> eyre::Result<()> {
         // make each elder propose a different thing
         for i in 0..net.procs.len() {
             let a_i = net.procs[i].id();
-            let vote = net.procs[i].propose(DummyProposal(i as u64))?;
+            let vote = net.procs[i].propose(i as u8)?;
             net.broadcast(a_i, vote);
         }
         net.drain_queued_packets()?;
@@ -121,12 +173,9 @@ fn test_handover_split_vote() -> eyre::Result<()> {
         // make sure they all reach the same conclusion
         let first_voters_value = net.consensus_value(0);
         for i in 0..nprocs {
-            println!(
-                "[TEST] checking elder {}'s consensus value: {:?}",
-                i,
-                net.consensus_value(i)
-            );
-            assert_eq!(net.consensus_value(i), first_voters_value);
+            let decision = net.consensus_value(i);
+            println!("[TEST] checking elder {i}'s consensus value: {decision:?}");
+            assert_eq!(decision, first_voters_value);
         }
     }
 
@@ -137,7 +186,7 @@ fn test_handover_split_vote() -> eyre::Result<()> {
 fn test_handover_round_robin_split_vote() -> eyre::Result<()> {
     let mut rng = StdRng::from_seed([0u8; 32]);
     for nprocs in 1..7 {
-        println!("[TEST] testing with {} elders", nprocs);
+        println!("[TEST] testing with {nprocs} elders");
 
         // make network of nprocs elders
         let mut net = Net::with_procs(((nprocs + 1) * 2 / 3).min(nprocs - 1), nprocs, &mut rng);
@@ -145,7 +194,7 @@ fn test_handover_round_robin_split_vote() -> eyre::Result<()> {
         // make each elder propose a different thing
         for i in 0..net.procs.len() {
             let a_i = net.procs[i].id();
-            let vote = net.procs[i].propose(DummyProposal(i as u64))?;
+            let vote = net.procs[i].propose(i as u8)?;
             net.broadcast(a_i, vote);
         }
 
@@ -165,17 +214,14 @@ fn test_handover_round_robin_split_vote() -> eyre::Result<()> {
         net.drain_queued_packets()?;
 
         // generate msc file
-        net.generate_msc(&format!("round_robin_split_vote_{}.msc", nprocs))?;
+        net.generate_msc(&format!("handover_round_robin_split_vote_{}.msc", nprocs))?;
 
         // make sure they all reach the same conclusion
         let max_proposed_value = nprocs - 1;
-        let expected_consensus_value = Some(DummyProposal(max_proposed_value as u64));
+        let expected_consensus_value = Some(max_proposed_value as u8);
         for i in 0..nprocs {
-            println!(
-                "[TEST] checking elder {}'s consensus value: {:?}",
-                i,
-                net.consensus_value(i)
-            );
+            let decision = net.consensus_value(i);
+            println!("[TEST] checking elder {i}'s consensus value: {decision:?}");
             assert_eq!(net.consensus_value(i), expected_consensus_value);
         }
     }
@@ -191,21 +237,18 @@ fn test_handover_simple_proposal() {
 
     // release a proposal
     let p0 = net.procs[0].id();
-    let vote = net.procs[0].propose(DummyProposal(42)).unwrap();
+    let vote = net.procs[0].propose(42).unwrap();
     net.broadcast(p0, vote);
     net.drain_queued_packets().unwrap();
     assert!(net.packets.is_empty());
 
-    net.generate_msc("simple_join.msc").unwrap();
+    net.generate_msc("handover_simple_join.msc").unwrap();
 
     // make sure they all reach the same conclusion
     let first_voters_value = net.consensus_value(0);
     for i in 0..n {
-        println!(
-            "[TEST] checking voter {}'s consensus value: {:?}",
-            i,
-            net.consensus_value(i)
-        );
-        assert_eq!(net.consensus_value(i), first_voters_value);
+        let decision = net.consensus_value(i);
+        println!("[TEST] checking voter {i}'s consensus value: {decision:?}");
+        assert_eq!(decision, first_voters_value);
     }
 }
